import os

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class DBHistoryMigration(MigrationABC):
    name = "1.0.0_DBHistoryMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def _servers(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `ServersHistory`
            (
                `ServerId`        BIGINT(20) NOT NULL,
                `DiscordServerId` BIGINT(20) NOT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _user_game_idents(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UserGameIdentsHistory`
            (
                `Id`             BIGINT(20)   NOT NULL,
                `UserId`         BIGINT(20)   NOT NULL,
                `GameServerId`   BIGINT(20)   NOT NULL,
                `Ident`          VARCHAR(255) NOT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _user_joined_game_server(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedGameServerHistory`
            (
                `Id`             BIGINT(20)  NOT NULL,
                `UserId`         BIGINT(20)  NOT NULL,
                `GameServerId`   BIGINT(20)  NOT NULL,
                `JoinedOn`       DATETIME(6) NOT NULL,
                `LeavedOn`       DATETIME(6) DEFAULT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _user_joined_server(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedServersHistory`
            (
                `JoinId`         BIGINT(20)  NOT NULL,
                `UserId`         BIGINT(20)  NOT NULL,
                `JoinedOn`       DATETIME(6) NOT NULL,
                `LeavedOn`       DATETIME(6) DEFAULT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _user_joined_voice_channel(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedVoiceChannelHistory`
            (
                `JoinId`           BIGINT(20)  NOT NULL,
                `UserId`           BIGINT(20)  NOT NULL,
                `DiscordChannelId` BIGINT(20)  NOT NULL,
                `JoinedOn`         DATETIME(6) NOT NULL,
                `LeavedOn`         DATETIME(6) DEFAULT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _user_message_count_per_hour(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UserMessageCountPerHourHistory`
            (
                `Id`             BIGINT(20)  NOT NULL,
                `Date`           DATETIME(6) NOT NULL,
                `Hour`           BIGINT(20)  DEFAULT NULL,
                `XPCount`        BIGINT(20)  DEFAULT NULL,
                `UserId`         BIGINT(20)  DEFAULT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _users(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UsersHistory`
            (
                `UserId`         BIGINT(20) NOT NULL,
                `DiscordId`      BIGINT(20) NOT NULL,
                `XP`             BIGINT(20) NOT NULL DEFAULT 0,
                `ServerId`       BIGINT(20)          DEFAULT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def _user_warnings(self):
        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `UserWarningsHistory`
            (
                `Id`             BIGINT(20)   NOT NULL,
                `Description`    VARCHAR(255) NOT NULL,
                `UserId`         BIGINT(20)   NOT NULL,
                `Author`         BIGINT(20)  DEFAULT NULL,
                `DateFrom`       DATETIME(6) NOT NULL,
                `DateTo`         DATETIME(6) NOT NULL
            );
            """
        )

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/api_keys.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auth_users.sql").read()
        )
        self._cursor.execute(
            open(
                f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auth_user_users_relation.sql"
            ).read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auto_role_rules.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auto_roles.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/clients.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/game_servers.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/known_users.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/levels.sql").read()
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `ApiKeysHistory`;")
        self._cursor.execute("DROP TABLE `AuthUsersHistory`;")
        self._cursor.execute("DROP TABLE `AuthUserUsersRelationsHistory`;")
        self._cursor.execute("DROP TABLE `AutoRoleRulesHistory`;")
        self._cursor.execute("DROP TABLE `AutoRolesHistory`;")
        self._cursor.execute("DROP TABLE `ClientsHistory`;")
        self._cursor.execute("DROP TABLE `GameServersHistory`;")
        self._cursor.execute("DROP TABLE `KnownUsersHistory`;")
        self._cursor.execute("DROP TABLE `LevelsHistory`;")
        self._cursor.execute("DROP TABLE `ServersHistory`;")
        self._cursor.execute("DROP TABLE `UserGameIdentsHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedGameServerHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedServersHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedVoiceChannelHistory`;")
        self._cursor.execute("DROP TABLE `UserMessageCountPerHourHistory`;")
        self._cursor.execute("DROP TABLE `UsersHistory`;")
        self._cursor.execute("DROP TABLE `UserWarningsHistory`;")
